@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1328,7 +1328,9 @@ void conv_code_converter_destroy(CodeConverter *conv)
 
 gchar *conv_convert(CodeConverter *conv, const gchar *inbuf)
 {
-	if (conv->code_conv_func != conv_noconv)
+	if (!inbuf)
+		return NULL;
+	else if (conv->code_conv_func != conv_noconv)
 		return conv->code_conv_func(inbuf, NULL);
 	else
 		return conv_iconv_strdup
@@ -1342,6 +1344,12 @@ gchar *conv_codeset_strdup_full(const gchar *inbuf,
 {
 	CodeConvFunc conv_func;
 
+	if (!inbuf) {
+		if (error)
+			*error = 0;
+		return NULL;
+	}
+
 	src_encoding = conv_get_fallback_for_private_encoding(src_encoding);
 
 	conv_func = conv_get_code_conv_func(src_encoding, dest_encoding);
@@ -1392,6 +1400,7 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 	case C_ISO_8859_13:
 	case C_ISO_8859_14:
 	case C_ISO_8859_15:
+	case C_ISO_8859_16:
 		break;
 	case C_ISO_2022_JP:
 	case C_ISO_2022_JP_2:
@@ -1400,12 +1409,20 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 			code_conv = conv_jistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_jistoeuc;
+		else if (dest_charset == C_SHIFT_JIS ||
+			 dest_charset == C_CP932)
+			code_conv = conv_jistosjis;
 		else if (dest_charset == C_UTF_8)
 			code_conv = conv_jistoutf8;
 		break;
 	case C_SHIFT_JIS:
+	case C_CP932:
 		if (dest_charset == C_AUTO)
 			code_conv = conv_sjistodisp;
+		else if (dest_charset == C_ISO_2022_JP   ||
+			 dest_charset == C_ISO_2022_JP_2 ||
+			 dest_charset == C_ISO_2022_JP_3)
+			code_conv = conv_sjistojis;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_sjistoeuc;
 		else if (dest_charset == C_UTF_8)
@@ -1428,6 +1445,9 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 			 dest_charset == C_ISO_2022_JP_2 ||
 			 dest_charset == C_ISO_2022_JP_3)
 			code_conv = conv_utf8tojis;
+		else if (dest_charset == C_SHIFT_JIS ||
+			 dest_charset == C_CP932)
+			code_conv = conv_utf8tosjis;
 		break;
 	default:
 		break;
@@ -1475,6 +1495,12 @@ gchar *conv_iconv_strdup_with_cd(const gchar *inbuf, iconv_t cd, gint *error)
 	size_t len;
 	gint error_ = 0;
 
+	if (!inbuf) {
+		if (error)
+			*error = 0;
+		return NULL;
+	}
+
 	inbuf_p = inbuf;
 	in_size = strlen(inbuf);
 	in_left = in_size;
@@ -1609,6 +1635,7 @@ static const struct {
 	{C_WINDOWS_874,		CS_WINDOWS_874},
 	{C_GEORGIAN_PS,		CS_GEORGIAN_PS},
 	{C_TCVN5712_1,		CS_TCVN5712_1},
+	{C_ISO_8859_16,		CS_ISO_8859_16},
 };
 
 static const struct {
@@ -2120,7 +2147,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			gint header_len, gboolean addr_field,
 			const gchar *out_encoding)
 {
-	const gchar *cur_encoding;
+	const gchar *src_encoding;
 	gint mimestr_len;
 	gchar *mimesep_enc;
 	gint left;
@@ -2138,18 +2165,20 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		mimesep_enc = "?Q?";
 	}
 
-	cur_encoding = CS_INTERNAL;
+	src_encoding = CS_INTERNAL;
 	if (!out_encoding)
 		out_encoding = conv_get_outgoing_charset_str();
 	if (!strcmp(out_encoding, CS_US_ASCII))
 		out_encoding = CS_ISO_8859_1;
 
-	mimestr_len = strlen(MIMESEP_BEGIN) + strlen(out_encoding) +
-		strlen(mimesep_enc) + strlen(MIMESEP_END);
+	mimestr_len = strlen(MIMESEP_BEGIN) + strlen(mimesep_enc) +
+		strlen(MIMESEP_END);
 
 	left = MAX_LINELEN - header_len;
 
 	while (*srcp) {
+		gboolean in_quote = FALSE;
+
 		LBREAK_IF_REQUIRED(left <= 0, TRUE);
 
 		while (g_ascii_isspace(*srcp)) {
@@ -2188,29 +2217,37 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			gchar *out_str;
 			gchar *enc_str;
 			const gchar *p = srcp;
+			const gchar *block_encoding = out_encoding;
 			gint out_str_len;
 			gint out_enc_str_len;
 			gint mime_block_len;
+			gint error = 0;
 			gboolean cont = FALSE;
 
 			while (*p != '\0') {
-				if (g_ascii_isspace(*p) &&
-				    !is_next_nonascii(p + 1))
-					break;
-				/* don't include parentheses in encoded
-				   strings */
-				if (addr_field && (*p == '(' || *p == ')'))
-					break;
+				if (*p == '"')
+					in_quote ^= TRUE;
+				else if (!in_quote) {
+					if (g_ascii_isspace(*p) &&
+					    !is_next_nonascii(p + 1))
+						break;
+					/* don't include parentheses in encoded
+					   strings */
+					if (addr_field &&
+					    (*p == '(' || *p == ')'))
+						break;
+				}
 
 				mb_len = g_utf8_skip[*(guchar *)p];
 
 				Xstrndup_a(part_str, srcp, cur_len + mb_len, );
-				out_str = conv_codeset_strdup
-					(part_str, cur_encoding, out_encoding);
-				if (!out_str) {
-					g_warning("conv_encode_header(): code conversion failed\n");
-					conv_unreadable_8bit(part_str);
+				out_str = conv_codeset_strdup_full
+					(part_str, src_encoding, block_encoding,
+					 &error);
+				if (!out_str || error != 0) {
+					g_warning("conv_encode_header(): code conversion failed. Keeping UTF-8.\n");
 					out_str = g_strdup(part_str);
+					block_encoding = CS_UTF_8;
 				}
 				out_str_len = strlen(out_str);
 
@@ -2223,26 +2260,32 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 				g_free(out_str);
 
-				if (mimestr_len + out_enc_str_len <= left) {
+				if (mimestr_len + strlen(block_encoding) + out_enc_str_len <= left) {
 					cur_len += mb_len;
 					p += mb_len;
 				} else if (cur_len == 0) {
 					LBREAK_IF_REQUIRED(1, FALSE);
+					if (*p == '"')
+						in_quote ^= TRUE;
 					continue;
 				} else {
 					cont = TRUE;
+					if (*p == '"')
+						in_quote ^= TRUE;
 					break;
 				}
 			}
 
 			if (cur_len > 0) {
 				Xstrndup_a(part_str, srcp, cur_len, );
-				out_str = conv_codeset_strdup
-					(part_str, cur_encoding, out_encoding);
-				if (!out_str) {
+				error = 0;
+				out_str = conv_codeset_strdup_full
+					(part_str, src_encoding, block_encoding,
+					 &error);
+				if (!out_str || error != 0) {
 					g_warning("conv_encode_header(): code conversion failed\n");
-					conv_unreadable_8bit(part_str);
 					out_str = g_strdup(part_str);
+					block_encoding = CS_UTF_8;
 				}
 				out_str_len = strlen(out_str);
 
@@ -2264,10 +2307,13 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				g_free(out_str);
 
 				/* output MIME-encoded string block */
-				mime_block_len = mimestr_len + strlen(enc_str);
+				mime_block_len = mimestr_len +
+					strlen(block_encoding) +
+					strlen(enc_str);
 				g_snprintf(destp, mime_block_len + 1,
 					   MIMESEP_BEGIN "%s%s%s" MIMESEP_END,
-					   out_encoding, mimesep_enc, enc_str);
+					   block_encoding, mimesep_enc,
+					   enc_str);
 				destp += mime_block_len;
 				srcp += cur_len;
 
