@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -140,19 +140,21 @@ void folder_remote_folder_destroy(RemoteFolder *rfolder)
 		session_destroy(rfolder->session);
 }
 
-#if 0
-Folder *mbox_folder_new(const gchar *name, const gchar *path)
+gint folder_scan_tree(Folder *folder)
 {
-	/* not yet implemented */
-	return NULL;
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(folder->klass->scan_tree != NULL, -1);
+
+	return folder->klass->scan_tree(folder);
 }
 
-Folder *maildir_folder_new(const gchar *name, const gchar *path)
+gint folder_create_tree(Folder *folder)
 {
-	/* not yet implemented */
-	return NULL;
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(folder->klass->create_tree != NULL, -1);
+
+	return folder->klass->create_tree(folder);
 }
-#endif
 
 FolderItem *folder_item_new(const gchar *name, const gchar *path)
 {
@@ -188,6 +190,7 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->auto_bcc = NULL;
 	item->auto_replyto = NULL;
 	item->mark_queue = NULL;
+	item->last_selected = 0;
 	item->data = NULL;
 
 	return item;
@@ -670,9 +673,6 @@ gchar *folder_item_get_identifier(FolderItem *item)
 
 	folder_id = folder_get_identifier(item->folder);
 	id = g_strconcat(folder_id, "/", item->path, NULL);
-#ifdef G_OS_WIN32
-	subst_char(id, G_DIR_SEPARATOR, '/');
-#endif
 	g_free(folder_id);
 
 	return id;
@@ -809,7 +809,7 @@ void folder_set_missing_folders(void)
 		    folder->queue && folder->trash)
 			continue;
 
-		if (folder->klass->create_tree(folder) < 0) {
+		if (folder_create_tree(folder) < 0) {
 			g_warning("%s: can't create the folder tree.\n",
 				  LOCAL_FOLDER(folder)->rootpath);
 			continue;
@@ -965,6 +965,22 @@ GSList *folder_item_get_msg_list(FolderItem *item, gboolean use_cache)
 	return folder->klass->get_msg_list(folder, item, use_cache);
 }
 
+GSList *folder_item_get_uncached_msg_list(FolderItem *item)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(item != NULL, NULL);
+	g_return_val_if_fail(item->folder->klass->get_uncached_msg_list != NULL,
+			     NULL);
+
+	folder = item->folder;
+
+	if (item->stype == F_VIRTUAL)
+		return NULL;
+
+	return folder->klass->get_uncached_msg_list(folder, item);
+}
+
 gchar *folder_item_fetch_msg(FolderItem *item, gint num)
 {
 	Folder *folder;
@@ -1276,9 +1292,12 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				stype = F_VIRTUAL;
 		} else if (!strcmp(attr->name, "name"))
 			name = attr->value;
-		else if (!strcmp(attr->name, "path"))
+		else if (!strcmp(attr->name, "path")) {
+#ifdef G_OS_WIN32
+			subst_char(attr->value, G_DIR_SEPARATOR, '/');
+#endif
 			path = attr->value;
-		else if (!strcmp(attr->name, "mtime"))
+		} else if (!strcmp(attr->name, "mtime"))
 			mtime = strtoul(attr->value, NULL, 10);
 		else if (!strcmp(attr->name, "new"))
 			new = atoi(attr->value);
