@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,10 @@
 #include <string.h>
 #include <errno.h>
 
+#ifdef G_OS_WIN32
+#  include <windows.h>
+#endif
+
 #undef MEASURE_TIME
 
 #include "folder.h"
@@ -52,6 +56,8 @@ static void     mh_folder_destroy	(Folder		*folder);
 static GSList  *mh_get_msg_list		(Folder		*folder,
 					 FolderItem	*item,
 					 gboolean	 use_cache);
+static GSList  *mh_get_uncached_msg_list(Folder		*folder,
+					 FolderItem	*item);
 static gchar   *mh_fetch_msg		(Folder		*folder,
 					 FolderItem	*item,
 					 gint		 num);
@@ -139,6 +145,7 @@ static FolderClass mh_class =
 	mh_create_tree,
 
 	mh_get_msg_list,
+	mh_get_uncached_msg_list,
 	mh_fetch_msg,
 	mh_get_msginfo,
 	mh_add_msg,
@@ -187,12 +194,13 @@ static void mh_folder_init(Folder *folder, const gchar *name, const gchar *path)
 	folder_local_folder_init(folder, name, path);
 }
 
-static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
-			       gboolean use_cache)
+static GSList *mh_get_msg_list_full(Folder *folder, FolderItem *item,
+				    gboolean use_cache, gboolean uncached_only)
 {
 	GSList *mlist;
 	GHashTable *msg_table;
 	time_t cur_mtime;
+	GSList *newlist = NULL;
 #ifdef MEASURE_TIME
 	GTimer *timer;
 #endif
@@ -214,7 +222,7 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 				item->cache_dirty = TRUE;
 		}
 	} else if (use_cache) {
-		GSList *newlist, *cur, *next;
+		GSList *cur, *next;
 		gboolean strict_cache_check = prefs_common.strict_cache_check;
 
 		if (item->stype == F_QUEUE || item->stype == F_DRAFT)
@@ -247,11 +255,14 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	} else {
 		mlist = mh_get_uncached_msgs(NULL, item);
 		item->cache_dirty = TRUE;
+		newlist = mlist;
 	}
 
 	procmsg_set_flags(mlist, item);
 
-	mlist = procmsg_sort_msg_list(mlist, item->sort_key, item->sort_type);
+	if (!uncached_only)
+		mlist = procmsg_sort_msg_list(mlist, item->sort_key,
+					      item->sort_type);
 
 	if (item->mark_queue)
 		item->mark_dirty = TRUE;
@@ -273,9 +284,40 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 			procmsg_write_flags_list(item, mlist);
 	}
 
+	if (uncached_only) {
+		GSList *cur;
+
+		if (newlist == NULL) {
+			procmsg_msg_list_free(mlist);
+			return NULL;
+		}
+		if (mlist == newlist)
+			return newlist;
+		for (cur = mlist; cur != NULL; cur = cur->next) {
+			if (cur->next == newlist) {
+				cur->next = NULL;
+				procmsg_msg_list_free(mlist);
+				return newlist;
+			}
+		}
+		procmsg_msg_list_free(mlist);
+		return NULL;
+	}
+
 	return mlist;
 }
 
+static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
+			       gboolean use_cache)
+{
+	return mh_get_msg_list_full(folder, item, use_cache, FALSE);
+}
+
+static GSList *mh_get_uncached_msg_list(Folder *folder, FolderItem *item)
+{
+	return mh_get_msg_list_full(folder, item, TRUE, TRUE);
+}
+
 static gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gchar *path;
@@ -505,9 +547,8 @@ static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 			g_warning(_("the src folder is identical to the dest.\n"));
 			continue;
 		}
-		debug_print("Moving message %s%c%d to %s ...\n",
-			    src->path, G_DIR_SEPARATOR, msginfo->msgnum,
-			    dest->path);
+		debug_print("Moving message %s/%d to %s ...\n",
+			    src->path, msginfo->msgnum, dest->path);
 
 		destfile = mh_get_new_msg_filename(dest);
 		if (!destfile) break;
@@ -627,9 +668,8 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 			g_warning(_("the src folder is identical to the dest.\n"));
 			continue;
 		}
-		debug_print(_("Copying message %s%c%d to %s ...\n"),
-			    msginfo->folder->path, G_DIR_SEPARATOR,
-			    msginfo->msgnum, dest->path);
+		debug_print(_("Copying message %s/%d to %s ...\n"),
+			    msginfo->folder->path, msginfo->msgnum, dest->path);
 
 		destfile = mh_get_new_msg_filename(dest);
 		if (!destfile) break;
@@ -735,13 +775,98 @@ static gint mh_close(Folder *folder, FolderItem *item)
 	return 0;
 }
 
+#ifdef G_OS_WIN32
+struct wfddata {
+	WIN32_FIND_DATAA wfda;
+	WIN32_FIND_DATAW wfdw;
+	DWORD file_attr;
+	gchar *file_name;
+};
+
+static HANDLE find_first_file(const gchar *path, struct wfddata *wfd)
+{
+	HANDLE hfind;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		if (path) {
+			gchar *wildcard_path;
+			wchar_t *wpath;
+
+			wildcard_path = g_strconcat(path, "\\*", NULL);
+			wpath = g_utf8_to_utf16(wildcard_path, -1,
+						NULL, NULL, NULL);
+			if (wpath) {
+				hfind = FindFirstFileW(wpath, &wfd->wfdw);
+				g_free(wpath);
+			} else
+				hfind = INVALID_HANDLE_VALUE;
+			g_free(wildcard_path);
+		} else
+			hfind = FindFirstFileW(L"*", &wfd->wfdw);
+
+		if (hfind != INVALID_HANDLE_VALUE) {
+			wfd->file_attr = wfd->wfdw.dwFileAttributes;
+			wfd->file_name = g_utf16_to_utf8(wfd->wfdw.cFileName, -1,
+							 NULL, NULL, NULL);
+		}
+	} else {
+		if (path) {
+			gchar *wildcard_path;
+			gchar *cp_path;
+
+			wildcard_path = g_strconcat(path, "\\*", NULL);
+			cp_path = g_locale_from_utf8(wildcard_path, -1,
+						     NULL, NULL, NULL);
+			if (cp_path) {
+				hfind = FindFirstFileA(cp_path, &wfd->wfda);
+				g_free(cp_path);
+			} else
+				hfind = INVALID_HANDLE_VALUE;
+			g_free(wildcard_path);
+		} else
+			hfind = FindFirstFileA("*", &wfd->wfda);
+
+		if (hfind != INVALID_HANDLE_VALUE) {
+			wfd->file_attr = wfd->wfda.dwFileAttributes;
+			wfd->file_name = g_locale_to_utf8(wfd->wfda.cFileName,
+							  -1, NULL, NULL, NULL);
+		}
+	}
+
+	return hfind;
+}
+
+static BOOL find_next_file(HANDLE hfind, struct wfddata *wfd)
+{
+	BOOL retval;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		retval = FindNextFileW(hfind, &wfd->wfdw);
+		if (retval) {
+			wfd->file_attr = wfd->wfdw.dwFileAttributes;
+			wfd->file_name = g_utf16_to_utf8(wfd->wfdw.cFileName, -1,
+							 NULL, NULL, NULL);
+		}
+	} else {
+		retval = FindNextFileA(hfind, &wfd->wfda);
+		if (retval) {
+			wfd->file_attr = wfd->wfda.dwFileAttributes;
+			wfd->file_name = g_locale_to_utf8(wfd->wfda.cFileName,
+							  -1, NULL, NULL, NULL);
+		}
+	}
+
+	return retval;
+}
+#endif
+
 static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 				gboolean count_sum)
 {
 	gchar *path;
 #ifdef G_OS_WIN32
-	GDir *dir;
-	const gchar *dir_name;
+	struct wfddata wfd;
+	HANDLE hfind;
 #else
 	DIR *dp;
 	struct dirent *d;
@@ -763,7 +888,7 @@ static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 	g_free(path);
 
 #ifdef G_OS_WIN32
-	if ((dir = g_dir_open(".", 0, NULL)) == NULL) {
+	if ((hfind = find_first_file(NULL, &wfd)) == INVALID_HANDLE_VALUE) {
 		g_warning("failed to open directory\n");
 #else
 	if ((dp = opendir(".")) == NULL) {
@@ -776,16 +901,25 @@ static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 		folder->ui_func(folder, item, folder->ui_func_data);
 
 #ifdef G_OS_WIN32
-	while ((dir_name = g_dir_read_name(dir)) != NULL) {
-		if ((num = to_number(dir_name)) > 0 &&
-		    g_file_test(dir_name, G_FILE_TEST_IS_REGULAR)) {
-			n_msg++;
-			if (max < num)
-				max = num;
+	do {
+		if ((wfd.file_attr &
+		     (FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_DEVICE)) == 0) {
+			if (wfd.file_name) {
+				if ((num = to_number(wfd.file_name)) > 0) {
+					n_msg++;
+					if (max < num)
+						max = num;
+				}
+			}
 		}
-	}
 
-	g_dir_close(dir);
+		if (wfd.file_name) {
+			g_free(wfd.file_name);
+			wfd.file_name = NULL;
+		}
+	} while (find_next_file(hfind, &wfd));
+
+	FindClose(hfind);
 #else
 	while ((d = readdir(dp)) != NULL) {
 		if ((num = to_number(d->d_name)) > 0 &&
@@ -821,7 +955,7 @@ static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
 	item->updated = TRUE;
 	item->mtime = 0;
 
-	debug_print(_("Last number in dir %s = %d\n"), item->path, max);
+	debug_print("Last number in dir %s = %d\n", item->path, max);
 	item->last_num = max;
 
 	return 0;
@@ -934,9 +1068,9 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 
 	g_free(fullpath);
 
+	/* path is a logical folder path */
 	if (parent->path)
-		path = g_strconcat(parent->path, G_DIR_SEPARATOR_S, name,
-				   NULL);
+		path = g_strconcat(parent->path, "/", name, NULL);
 	else
 		path = g_strdup(name);
 	new_item = folder_item_new(name, path);
@@ -1037,17 +1171,15 @@ static gint mh_move_folder_real(Folder *folder, FolderItem *item,
 		g_node_append(new_parent->node, item->node);
 		item->parent = new_parent;
 		if (new_parent->path != NULL) {
-			newpath = g_strconcat(new_parent->path,
-					      G_DIR_SEPARATOR_S, utf8_name,
+			newpath = g_strconcat(new_parent->path, "/", utf8_name,
 					      NULL);
 			g_free(utf8_name);
 		} else
 			newpath = utf8_name;
 	} else {
-		if (strchr(item->path, G_DIR_SEPARATOR) != NULL) {
+		if (strchr(item->path, '/') != NULL) {
 			dirname = g_dirname(item->path);
-			newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S,
-					      utf8_name, NULL);
+			newpath = g_strconcat(dirname, "/", utf8_name, NULL);
 			g_free(dirname);
 			g_free(utf8_name);
 		} else
@@ -1301,13 +1433,14 @@ static void mh_scan_tree_recursive(FolderItem *item)
 {
 	Folder *folder;
 #ifdef G_OS_WIN32
-	GDir *dir;
+	struct wfddata wfd;
+	HANDLE hfind;
 #else
 	DIR *dp;
 	struct dirent *d;
+	struct stat s;
 #endif
 	const gchar *dir_name;
-	struct stat s;
 	gchar *fs_path;
 	gchar *entry;
 	gchar *utf8entry;
@@ -1317,6 +1450,9 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
+	if (item->stype == F_VIRTUAL)
+		return;
+
 	folder = item->folder;
 
 	fs_path = item->path ?
@@ -1325,8 +1461,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	if (!fs_path)
 		fs_path = g_strdup(item->path);
 #ifdef G_OS_WIN32
-	dir = g_dir_open(fs_path, 0, NULL);
-	if (!dir) {
+	hfind = find_first_file(fs_path, &wfd);
+	if (hfind == INVALID_HANDLE_VALUE) {
 		g_warning("failed to open directory: %s\n", fs_path);
 		g_free(fs_path);
 		return;
@@ -1348,48 +1484,54 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		folder->ui_func(folder, item, folder->ui_func_data);
 
 #ifdef G_OS_WIN32
-	while ((dir_name = g_dir_read_name(dir)) != NULL) {
+	do {
+		if (!wfd.file_name) continue;
+		if (wfd.file_name[0] == '.') {
+			g_free(wfd.file_name);
+			wfd.file_name = NULL;
+			continue;
+		}
+		dir_name = utf8name = wfd.file_name;
+		wfd.file_name = NULL;
 #else
 	while ((d = readdir(dp)) != NULL) {
 		dir_name = d->d_name;
-#endif
 		if (dir_name[0] == '.') continue;
 
 		utf8name = g_filename_to_utf8(dir_name, -1, NULL, NULL, NULL);
 		if (!utf8name)
 			utf8name = g_strdup(dir_name);
+#endif
 
-		if (item->path)
-			utf8entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
-						utf8name, NULL);
-		else
+		if (item->path) {
+			utf8entry = g_strconcat(item->path, "/", utf8name,
+						NULL);
+		} else
 			utf8entry = g_strdup(utf8name);
 		entry = g_filename_from_utf8(utf8entry, -1, NULL, NULL, NULL);
 		if (!entry)
 			entry = g_strdup(utf8entry);
+#ifdef G_OS_WIN32
+		subst_char(entry, '/', G_DIR_SEPARATOR);
+#endif
 
 		if (
-#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
+#ifdef G_OS_WIN32
+			(wfd.file_attr & FILE_ATTRIBUTE_DIRECTORY) != 0
+#else
+#if HAVE_DIRENT_D_TYPE
 			d->d_type == DT_DIR ||
-			(d->d_type == DT_UNKNOWN &&
+			((d->d_type == DT_UNKNOWN || d->d_type == DT_LNK) &&
 #endif
 			g_stat(entry, &s) == 0 && S_ISDIR(s.st_mode)
-#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
+#if HAVE_DIRENT_D_TYPE
 			)
 #endif
+#endif /* G_OS_WIN32 */
 		   ) {
 			FolderItem *new_item = NULL;
 			GNode *node;
 
-#if 0
-			if (mh_is_maildir(entry)) {
-				g_free(entry);
-				g_free(utf8entry);
-				g_free(utf8name);
-				continue;
-			}
-#endif
-
 #ifndef G_OS_WIN32
 			if (g_utf8_validate(utf8name, -1, NULL) == FALSE) {
 				g_warning(_("Directory name\n"
@@ -1410,17 +1552,14 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			node = item->node;
 			for (node = node->children; node != NULL; node = node->next) {
 				FolderItem *cur_item = FOLDER_ITEM(node->data);
-#ifdef G_OS_WIN32
-				if (!path_cmp(cur_item->path, utf8entry)) {
-#else
 				if (!strcmp2(cur_item->path, utf8entry)) {
-#endif
 					new_item = cur_item;
 					break;
 				}
 			}
 			if (!new_item) {
-				debug_print("new folder '%s' found.\n", entry);
+				debug_print("new folder '%s' found.\n",
+					    utf8entry);
 				new_item = folder_item_new(utf8name, utf8entry);
 				folder_item_append(item, new_item);
 			}
@@ -1455,10 +1594,14 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		g_free(entry);
 		g_free(utf8entry);
 		g_free(utf8name);
+#ifdef G_OS_WIN32
+	} while (find_next_file(hfind, &wfd));
+#else
 	}
+#endif
 
 #ifdef G_OS_WIN32
-	g_dir_close(dir);
+	FindClose(hfind);
 #else
 	closedir(dp);
 #endif
@@ -1497,12 +1640,11 @@ static gboolean mh_rename_folder_func(GNode *node, gpointer data)
 	}
 
 	base = item->path + oldpathlen;
-	while (*base == G_DIR_SEPARATOR) base++;
+	while (*base == '/') base++;
 	if (*base == '\0')
 		new_itempath = g_strdup(newpath);
 	else
-		new_itempath = g_strconcat(newpath, G_DIR_SEPARATOR_S, base,
-					   NULL);
+		new_itempath = g_strconcat(newpath, "/", base, NULL);
 	g_free(item->path);
 	item->path = new_itempath;
 
