@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -83,6 +83,8 @@ static PrefParam param[] = {
 	 &prefs_common.reply_account_autosel, P_BOOL},
 	{"default_reply_list", "TRUE", &prefs_common.default_reply_list,
 	 P_BOOL},
+	{"inherit_recipient_on_self_reply", "FALSE",
+	 &prefs_common.inherit_recipient_on_self_reply, P_BOOL},
 
 	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL},
 
@@ -124,10 +126,15 @@ static PrefParam param[] = {
 	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL},
 	{"bold_unread", "TRUE", &prefs_common.bold_unread, P_BOOL},
 
-	{"toolbar_style", "3", &prefs_common.toolbar_style, P_ENUM},
+	{"toolbar_style", "4", &prefs_common.toolbar_style, P_ENUM},
 	{"show_searchbar", "TRUE", &prefs_common.show_searchbar, P_BOOL},
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL},
 
+	{"main_toolbar_setting", NULL, &prefs_common.main_toolbar_setting,
+	 P_STRING},
+	{"compose_toolbar_setting", NULL,
+	 &prefs_common.compose_toolbar_setting, P_STRING},
+
 	{"summary_col_show_mark", "TRUE",
 	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL},
 	{"summary_col_show_unread", "TRUE",
@@ -320,9 +327,13 @@ static PrefParam param[] = {
 	 &prefs_common.always_show_msg, P_BOOL},
 	{"open_unread_on_enter", "FALSE", &prefs_common.open_unread_on_enter,
 	 P_BOOL},
+	{"remember_last_selected", "FALSE",
+	 &prefs_common.remember_last_selected, P_BOOL},
 	{"mark_as_read_on_new_window", "FALSE",
 	 &prefs_common.mark_as_read_on_new_window, P_BOOL},
 	{"open_inbox_on_inc", "FALSE", &prefs_common.open_inbox_on_inc, P_BOOL},
+	{"open_inbox_on_startup", "FALSE", &prefs_common.open_inbox_on_startup,
+	 P_BOOL},
 	{"immediate_execution", "TRUE", &prefs_common.immediate_exec, P_BOOL},
 
 #ifdef G_OS_WIN32
@@ -332,6 +343,13 @@ static PrefParam param[] = {
 #endif
 
 	{"show_trayicon", "TRUE", &prefs_common.show_trayicon, P_BOOL},
+	{"minimize_to_tray", "FALSE", &prefs_common.minimize_to_tray, P_BOOL},
+#ifdef G_OS_WIN32
+	{"toggle_window_on_trayicon_click", "FALSE",
+#else
+	{"toggle_window_on_trayicon_click", "TRUE",
+#endif
+	 &prefs_common.toggle_window_on_trayicon_click, P_BOOL},
 
 	/* Other */
 	{"receive_dialog_mode", "1", &prefs_common.recv_dialog_mode, P_ENUM},
@@ -359,11 +377,6 @@ static PrefParam param[] = {
 	{"uri_open_command", DEFAULT_BROWSER_CMD, &prefs_common.uri_cmd,
 #endif
 	 P_STRING},
-#ifdef G_OS_WIN32
-	{"print_command", NULL, &prefs_common.print_cmd, P_STRING},
-#else
-	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING},
-#endif
 #ifdef G_OS_WIN32
 	{"ext_editor_command", "notepad '%s'", &prefs_common.ext_editor_cmd,
 #else
@@ -371,6 +384,13 @@ static PrefParam param[] = {
 #endif
 	 P_STRING},
 
+	{"use_print_command", "FALSE", &prefs_common.use_print_cmd, P_BOOL},
+#ifdef G_OS_WIN32
+	{"print_command", NULL, &prefs_common.print_cmd, P_STRING},
+#else
+	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING},
+#endif
+
 	{"use_ext_inc", "FALSE", &prefs_common.use_extinc, P_BOOL},
 	{"ext_inc_path", DEFAULT_INC_PATH, &prefs_common.extinc_cmd, P_STRING},
 	{"use_ext_sendmail", "FALSE", &prefs_common.use_extsend, P_BOOL},
@@ -386,6 +406,11 @@ static PrefParam param[] = {
 };
 
 
+PrefsCommon *prefs_common_get(void)
+{
+	return &prefs_common;
+}
+
 PrefParam *prefs_common_get_params(void)
 {
 	return param;
@@ -401,15 +426,6 @@ void prefs_common_read_config(void)
 
 	prefs_read_config(param, "Common", path, NULL);
 
-#ifdef G_OS_WIN32
-	if (!is_file_exist(path)) {
-		if (conv_is_ja_locale()) {
-			g_free(prefs_common.textfont);
-			prefs_common.textfont = g_strdup("MS Gothic 12");
-		}
-	}
-#endif
-
 	g_free(path);
 
 	prefs_common.online_mode = TRUE;
