@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -84,6 +84,8 @@ struct _PrefsCommon
 	gint autosave_itv;
 	gboolean reply_account_autosel;
 	gboolean default_reply_list;
+	gboolean inherit_recipient_on_self_reply;
+
 	gboolean show_ruler;
 
 	/* Quote */
@@ -116,6 +118,9 @@ struct _PrefsCommon
 	gboolean show_searchbar;
 	gboolean show_statusbar;
 
+	gchar *main_toolbar_setting;
+	gchar *compose_toolbar_setting;
+
 	/* Summary columns visibility, position and size */
 	gboolean summary_col_visible[N_SUMMARY_COLS];
 	gint summary_col_pos[N_SUMMARY_COLS];
@@ -230,11 +235,15 @@ struct _PrefsCommon
 	gboolean sep_msg;
 	gboolean always_show_msg;
 	gboolean open_unread_on_enter;
+	gboolean remember_last_selected;
 	gboolean mark_as_read_on_new_window;
 	gboolean open_inbox_on_inc;
+	gboolean open_inbox_on_startup;
 	gboolean immediate_exec;
 	gboolean comply_gnome_hig;
 	gboolean show_trayicon;
+	gboolean minimize_to_tray;
+	gboolean toggle_window_on_trayicon_click;
 
 	/* Other */
 	RecvDialogMode recv_dialog_mode;
@@ -252,9 +261,11 @@ struct _PrefsCommon
 
 	/* External commands */
 	gchar *uri_cmd;
-	gchar *print_cmd;
 	gchar *ext_editor_cmd;
 
+	gboolean use_print_cmd;
+	gchar *print_cmd;
+
 	gboolean use_extinc;
 	gchar *extinc_cmd;
 	gboolean use_extsend;
@@ -278,6 +289,8 @@ struct _PrefsCommon
 
 extern PrefsCommon prefs_common;
 
+PrefsCommon *prefs_common_get		(void);
+
 PrefParam *prefs_common_get_params	(void);
 
 void prefs_common_read_config		(void);
