@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,7 @@
 #  include <wchar.h>
 #  include <direct.h>
 #  include <io.h>
+#  include <shlobj.h>
 #endif
 
 #include "utils.h"
@@ -467,13 +468,19 @@ gint str_find_format_times(const gchar *haystack, gchar ch)
 gboolean is_next_nonascii(const gchar *s)
 {
 	const gchar *p;
+	gboolean in_quote = FALSE;
 
 	/* skip head space */
 	for (p = s; *p != '\0' && g_ascii_isspace(*p); p++)
 		;
-	for (; *p != '\0' && !g_ascii_isspace(*p); p++) {
-		if (*(guchar *)p > 127 || *(guchar *)p < 32)
+	while (*p != '\0') {
+		if (!in_quote && g_ascii_isspace(*p))
+			break;
+		if (*p == '"')
+			in_quote ^= TRUE;
+		else if (*(guchar *)p > 127 || *(guchar *)p < 32)
 			return TRUE;
+		++p;
 	}
 
 	return FALSE;
@@ -481,12 +488,19 @@ gboolean is_next_nonascii(const gchar *s)
 
 gint get_next_word_len(const gchar *s)
 {
+	const gchar *p = s;
 	gint len = 0;
+	gboolean in_quote = FALSE;
 
-	for (; *s != '\0' && !g_ascii_isspace(*s); s++, len++)
-		;
+	while (*p != '\0') {
+		if (!in_quote && g_ascii_isspace(*p))
+			break;
+		if (*p == '"')
+			in_quote ^= TRUE;
+		++p;
+	}
 
-	return len;
+	return p - s;
 }
 
 /* compare subjects */
@@ -1006,7 +1020,7 @@ GList *add_history(GList *list, const gchar *str)
 		last = g_list_last(list);
 		if (last) {
 			g_free(last->data);
-			g_list_remove(list, last->data);
+			list = g_list_remove(list, last->data);
 		}
 	}
 
@@ -1341,7 +1355,6 @@ gchar **strsplit_parenthesis(const gchar *str, gchar op, gchar cl,
 	str_array = g_new(gchar*, n);
 
 	i = n - 1;
-
 	str_array[i--] = NULL;
 	for (slist = string_list; slist; slist = slist->next)
 		str_array[i--] = slist->data;
@@ -1401,7 +1414,79 @@ gchar **strsplit_with_quote(const gchar *str, const gchar *delim,
 	str_array = g_new(gchar*, n);
 
 	i = n - 1;
+	str_array[i--] = NULL;
+	for (slist = string_list; slist; slist = slist->next)
+		str_array[i--] = slist->data;
 
+	g_slist_free(string_list);
+
+	return str_array;
+}
+
+gchar **strsplit_csv(const gchar *str, gchar delim, gint max_tokens)
+{
+	GSList *string_list = NULL, *slist;
+	gchar **str_array, *s, *new_str;
+	gchar *tmp, *tmpp, *p;
+	guint i, n = 1, len;
+
+	g_return_val_if_fail(str != NULL, NULL);
+
+	if (max_tokens < 1)
+		max_tokens = G_MAXINT;
+
+	s = strchr_with_skip_quote(str, '"', delim);
+	if (s) {
+		do {
+			len = s - str;
+			tmpp = tmp = g_strndup(str, len);
+
+			if (tmp[0] == '"' && tmp[len - 1] == tmp[0]) {
+				tmp[len - 1] = '\0';
+				++tmpp;
+				p = new_str = g_malloc(len - 1);
+				while (*tmpp) {
+					if (*tmpp == '"' && *(tmpp + 1) == '"')
+						++tmpp;
+					*p++ = *tmpp++;
+				}
+				*p = '\0';
+				g_free(tmp);
+			} else
+				new_str = tmp;
+
+			string_list = g_slist_prepend(string_list, new_str);
+			n++;
+			str = s + 1;
+			s = strchr_with_skip_quote(str, '"', delim);
+		} while (--max_tokens && s);
+	}
+
+	if (*str) {
+		len = strlen(str);
+		tmpp = tmp = g_strdup(str);
+
+		if (tmp[0] == '"' && tmp[len - 1] == tmp[0]) {
+			tmp[len - 1] = '\0';
+			++tmpp;
+			p = new_str = g_malloc(len - 1);
+			while (*tmpp) {
+				if (*tmpp == '"' && *(tmpp + 1) == '"')
+					++tmpp;
+				*p++ = *tmpp++;
+			}
+			*p = '\0';
+			g_free(tmp);
+		} else
+			new_str = tmp;
+
+		string_list = g_slist_prepend(string_list, new_str);
+		n++;
+	}
+
+	str_array = g_new(gchar*, n);
+
+	i = n - 1;
 	str_array[i--] = NULL;
 	for (slist = string_list; slist; slist = slist->next)
 		str_array[i--] = slist->data;
@@ -1711,7 +1796,7 @@ gchar *uriencode_for_filename(const gchar *filename)
 }
 
 gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
-		     gchar **subject, gchar **body)
+		     gchar **subject, gchar **inreplyto, gchar **body)
 {
 	gchar *tmp_mailto;
 	gchar *p;
@@ -1727,8 +1812,10 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 		p++;
 	}
 
-	if (to && !*to)
-		*to = g_strdup(tmp_mailto);
+	if (to && !*to) {
+		*to = g_malloc(strlen(tmp_mailto) + 1);
+		decode_uri(*to, tmp_mailto);
+	}
 
 	while (p) {
 		gchar *field, *value;
@@ -1751,13 +1838,19 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 		if (*value == '\0') continue;
 
 		if (cc && !*cc && !g_ascii_strcasecmp(field, "cc")) {
-			*cc = g_strdup(value);
+			*cc = g_malloc(strlen(value) + 1);
+			decode_uri(*cc, value);
 		} else if (bcc && !*bcc && !g_ascii_strcasecmp(field, "bcc")) {
-			*bcc = g_strdup(value);
+			*bcc = g_malloc(strlen(value) + 1);
+			decode_uri(*bcc, value);
 		} else if (subject && !*subject &&
 			   !g_ascii_strcasecmp(field, "subject")) {
 			*subject = g_malloc(strlen(value) + 1);
 			decode_uri(*subject, value);
+		} else if (inreplyto && !*inreplyto &&
+			   !g_ascii_strcasecmp(field, "in-reply-to")) {
+			*inreplyto = g_malloc(strlen(value) + 1);
+			decode_uri(*inreplyto, value);
 		} else if (body && !*body &&
 			   !g_ascii_strcasecmp(field, "body")) {
 			*body = g_malloc(strlen(value) + 1);
@@ -1815,6 +1908,28 @@ const gchar *get_startup_dir(void)
 	return startup_dir;
 }
 
+#ifdef G_OS_WIN32
+static gchar *get_win32_special_folder_path(gint nfolder)
+{
+	gchar *folder = NULL;
+	HRESULT hr;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		wchar_t path[MAX_PATH + 1];
+		hr = SHGetFolderPathW(NULL, nfolder, NULL, 0, path);
+		if (hr == S_OK)
+			folder = g_utf16_to_utf8(path, -1, NULL, NULL, NULL);
+	} else {
+		gchar path[MAX_PATH + 1];
+		hr = SHGetFolderPathA(NULL, nfolder, NULL, 0, path);
+		if (hr == S_OK)
+			folder = g_locale_to_utf8(path, -1, NULL, NULL, NULL);
+	}
+
+	return folder;
+}
+#endif
+
 const gchar *get_home_dir(void)
 {
 #ifdef G_OS_WIN32
@@ -1832,19 +1947,38 @@ const gchar *get_home_dir(void)
 #endif
 }
 
+const gchar *get_document_dir(void)
+{
+#ifdef G_OS_WIN32
+	static const gchar *document_dir = NULL;
+	HRESULT hr;
+
+	if (!document_dir) {
+		document_dir = get_win32_special_folder_path(CSIDL_PERSONAL);
+		if (!document_dir)
+			document_dir = get_home_dir();
+	}
+
+	return document_dir;
+#else
+	return get_home_dir();
+#endif
+}
+
 const gchar *get_rc_dir(void)
 {
 	if (!rc_dir) {
 #ifdef G_OS_WIN32
-		const gchar *appdata;
+		gchar *appdata;
 
-		appdata = g_getenv("APPDATA");
+		appdata = get_win32_special_folder_path(CSIDL_APPDATA);
 		if (appdata)
 			rc_dir = g_strconcat(appdata, G_DIR_SEPARATOR_S,
 					     RC_DIR, NULL);
 		else
 			rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					     RC_DIR, NULL);
+		g_free(appdata);
 #else
 		rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 				     RC_DIR, NULL);
@@ -1948,7 +2082,6 @@ gchar *get_tmp_file(void)
 
 const gchar *get_domain_name(void)
 {
-#ifdef G_OS_UNIX
 	static gchar *domain_name = NULL;
 
 	if (!domain_name) {
@@ -1969,12 +2102,14 @@ const gchar *get_domain_name(void)
 		}
 
 		debug_print("domain name = %s\n", domain_name);
+		if (is_next_nonascii(domain_name)) {
+			g_warning("invalid domain name: %s\n", domain_name);
+			g_free(domain_name);
+			domain_name = "unknown";
+		}
 	}
 
 	return domain_name;
-#else
-	return "unknown";
-#endif
 }
 
 off_t get_file_size(const gchar *file)
@@ -2044,18 +2179,21 @@ off_t get_left_file_size(FILE *fp)
 
 gboolean file_exist(const gchar *file, gboolean allow_fifo)
 {
-	struct stat s;
-
 	if (file == NULL)
 		return FALSE;
 
-	if (g_stat(file, &s) < 0) {
-		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
-		return FALSE;
-	}
+	if (allow_fifo) {
+		struct stat s;
 
-	if (S_ISREG(s.st_mode) || (allow_fifo && S_ISFIFO(s.st_mode)))
-		return TRUE;
+		if (g_stat(file, &s) < 0) {
+			if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
+			return FALSE;
+		}
+		if (S_ISREG(s.st_mode) || S_ISFIFO(s.st_mode))
+			return TRUE;
+	} else {
+		return g_file_test(file, G_FILE_TEST_IS_REGULAR);
+	}
 
 	return FALSE;
 }
@@ -2369,8 +2507,15 @@ static gint remove_dir_recursive_real(const gchar *dir)
 	g_free(prev_dir);
 
 	if (g_rmdir(dir) < 0) {
-		FILE_OP_ERROR(dir, "rmdir");
-		return -1;
+		if (ENOTDIR == errno) {
+			if (g_unlink(dir) < 0) {
+				FILE_OP_ERROR(dir, "unlink");
+				return -1;
+			}
+		} else {
+			FILE_OP_ERROR(dir, "rmdir");
+			return -1;
+		}
 	}
 
 	return 0;
@@ -2410,7 +2555,7 @@ leave:
 
 gint rename_force(const gchar *oldpath, const gchar *newpath)
 {
-#ifndef G_OS_UNIX
+#if !defined(G_OS_UNIX) && !GLIB_CHECK_VERSION(2, 9, 1)
 	if (!is_file_entry_exist(oldpath)) {
 		errno = ENOENT;
 		return -1;
@@ -2425,29 +2570,37 @@ gint rename_force(const gchar *oldpath, const gchar *newpath)
 
 gint copy_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 {
-	FILE *src_fp, *dest_fp;
+	gint srcfd, destfd;
 	gint n_read;
-	gchar buf[BUFSIZ];
+	gchar buf[BUFFSIZE];
 	gchar *dest_bak = NULL;
 	gboolean err = FALSE;
 
-	if ((src_fp = g_fopen(src, "rb")) == NULL) {
-		FILE_OP_ERROR(src, "fopen");
+#ifdef G_OS_WIN32
+	if ((srcfd = g_open(src, O_RDONLY | O_BINARY, 0600)) < 0) {
+#else
+	if ((srcfd = g_open(src, O_RDONLY, 0600)) < 0) {
+#endif
+		FILE_OP_ERROR(src, "open");
 		return -1;
 	}
 	if (is_file_exist(dest)) {
 		dest_bak = g_strconcat(dest, ".bak", NULL);
 		if (rename_force(dest, dest_bak) < 0) {
 			FILE_OP_ERROR(dest, "rename");
-			fclose(src_fp);
+			close(srcfd);
 			g_free(dest_bak);
 			return -1;
 		}
 	}
 
-	if ((dest_fp = g_fopen(dest, "wb")) == NULL) {
-		FILE_OP_ERROR(dest, "fopen");
-		fclose(src_fp);
+#ifdef G_OS_WIN32
+	if ((destfd = g_open(dest, O_WRONLY | O_CREAT | O_BINARY, 0600)) < 0) {
+#else
+	if ((destfd = g_open(dest, O_WRONLY | O_CREAT, 0600)) < 0) {
+#endif
+		FILE_OP_ERROR(dest, "open");
+		close(srcfd);
 		if (dest_bak) {
 			if (rename_force(dest_bak, dest) < 0)
 				FILE_OP_ERROR(dest_bak, "rename");
@@ -2456,37 +2609,33 @@ gint copy_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 		return -1;
 	}
 
-	if (change_file_mode_rw(dest_fp, dest) < 0) {
-		FILE_OP_ERROR(dest, "chmod");
-		g_warning(_("can't change file mode\n"));
-	}
+	while ((n_read = read(srcfd, buf, sizeof(buf))) > 0) {
+		gchar *p = buf;
+		const gchar *endp = buf + n_read;
+		gint n_write;
 
-	while ((n_read = fread(buf, sizeof(gchar), sizeof(buf), src_fp)) > 0) {
-		if (n_read < sizeof(buf) && ferror(src_fp))
-			break;
-		if (fwrite(buf, n_read, 1, dest_fp) < 1) {
-			g_warning(_("writing to %s failed.\n"), dest);
-			fclose(dest_fp);
-			fclose(src_fp);
-			g_unlink(dest);
-			if (dest_bak) {
-				if (rename_force(dest_bak, dest) < 0)
-					FILE_OP_ERROR(dest_bak, "rename");
-				g_free(dest_bak);
+		while (p < endp) {
+			if ((n_write = write(destfd, p, endp - p)) < 0) {
+				g_warning(_("writing to %s failed.\n"), dest);
+				close(destfd);
+				close(srcfd);
+				g_unlink(dest);
+				if (dest_bak) {
+					if (rename_force(dest_bak, dest) < 0)
+						FILE_OP_ERROR(dest_bak, "rename");
+					g_free(dest_bak);
+				}
+				return -1;
 			}
-			return -1;
+			p += n_write;
 		}
 	}
 
-	if (ferror(src_fp)) {
-		FILE_OP_ERROR(src, "fread");
-		err = TRUE;
-	}
-	fclose(src_fp);
-	if (fclose(dest_fp) == EOF) {
-		FILE_OP_ERROR(dest, "fclose");
+	if (close(destfd) < 0) {
+		FILE_OP_ERROR(dest, "close");
 		err = TRUE;
 	}
+	close(srcfd);
 
 	if (err) {
 		g_unlink(dest);
@@ -2540,7 +2689,7 @@ gint copy_dir(const gchar *src, const gchar *dest)
 
 gint move_file(const gchar *src, const gchar *dest, gboolean overwrite)
 {
-	if (overwrite == FALSE && is_file_exist(dest)) {
+	if (overwrite == FALSE && is_file_entry_exist(dest)) {
 		g_warning("move_file(): file %s already exists.", dest);
 		return -1;
 	}
@@ -3117,10 +3266,47 @@ gchar *generate_mime_boundary(const gchar *prefix)
 
 gint change_file_mode_rw(FILE *fp, const gchar *file)
 {
-#if HAVE_FCHMOD
-	return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
+#ifdef G_OS_WIN32
+	DWORD attr;
+	BOOL retval;
+
+	if (G_WIN32_HAVE_WIDECHAR_API()) {
+		wchar_t *wpath;
+
+		wpath = g_utf8_to_utf16(file, -1, NULL, NULL, NULL);
+		if (wpath == NULL)
+			return -1;
+
+		attr = GetFileAttributesW(wpath);
+		retval = SetFileAttributesW
+			(wpath, attr & ~(FILE_ATTRIBUTE_READONLY|FILE_ATTRIBUTE_HIDDEN));
+
+		g_free(wpath);
+	} else {
+		gchar *cp_path;
+
+		cp_path = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
+		if (cp_path == NULL)
+			return -1;
+
+		attr = GetFileAttributesA(cp_path);
+		retval = SetFileAttributesA
+			(cp_path, attr & ~(FILE_ATTRIBUTE_READONLY|FILE_ATTRIBUTE_HIDDEN));
+
+		g_free(cp_path);
+	}
+
+	if (retval)
+		return 0;
+	else
+		return -1;
 #else
-	return g_chmod(file, S_IRUSR|S_IWUSR);
+#if HAVE_FCHMOD
+	if (fp)
+		return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
+	else
+#endif
+		return g_chmod(file, S_IRUSR|S_IWUSR);
 #endif
 }
 
@@ -3238,6 +3424,8 @@ FILE *my_tmpfile(void)
 	tmpdir = get_tmp_dir();
 	tmplen = strlen(tmpdir);
 	progname = g_get_prgname();
+	if (!progname)
+		progname = "sylph";
 	proglen = strlen(progname);
 	Xalloca(fname, tmplen + 1 + proglen + sizeof(suffix),
 		return tmpfile());
@@ -3951,7 +4139,7 @@ void log_print(const gchar *format, ...)
 	g_vsnprintf(buf + TIME_LEN, BUFFSIZE, format, args);
 	va_end(args);
 
-	if (debug_mode) fputs(buf, stdout);
+	if (debug_mode) g_print("%s", buf);
 	log_print_ui_func(buf);
 	if (log_fp) {
 		fputs(buf, log_fp);
