@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -111,15 +111,24 @@ static void about_create(void)
 #if HAVE_SYS_UTSNAME_H
 	uname(&utsbuf);
 	g_snprintf(buf, sizeof(buf),
-		   "GTK+ version %d.%d.%d\n"
+		   "GTK+ %d.%d.%d / GLib %d.%d.%d\n"
 		   "Operating System: %s %s (%s)",
 		   gtk_major_version, gtk_minor_version, gtk_micro_version,
+		   glib_major_version, glib_minor_version, glib_micro_version,
 		   utsbuf.sysname, utsbuf.release, utsbuf.machine);
+#elif defined(G_OS_WIN32)
+	g_snprintf(buf, sizeof(buf),
+		   "GTK+ %d.%d.%d / GLib %d.%d.%d\n"
+		   "Operating System: %s",
+		   gtk_major_version, gtk_minor_version, gtk_micro_version,
+		   glib_major_version, glib_minor_version, glib_micro_version,
+		   "Win32");
 #else
 	g_snprintf(buf, sizeof(buf),
-		   "GTK+ version %d.%d.%d\n"
-		   "Operating System: Windoze",
-		   gtk_major_version, gtk_minor_version, gtk_micro_version);
+		   "GTK+ %d.%d.%d / GLib %d.%d.%d\n"
+		   "Operating System: unknown",
+		   gtk_major_version, gtk_minor_version, gtk_micro_version,
+		   glib_major_version, glib_minor_version, glib_micro_version);
 #endif
 
 	label = gtk_label_new(buf);
@@ -152,6 +161,12 @@ static void about_create(void)
 #endif
 #if USE_JPILOT
 		   " JPilot"
+#endif
+#if USE_GTKSPELL
+		   " GtkSpell"
+#endif
+#if USE_ONIGURUMA
+		   " Oniguruma"
 #endif
 	"");
 
@@ -160,7 +175,7 @@ static void about_create(void)
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 	label = gtk_label_new
-		("Copyright (C) 1999-2005 Hiroyuki Yamamoto <hiro-y@kcn.ne.jp>");
+		("Copyright (C) 1999-2007 Hiroyuki Yamamoto <hiro-y@kcn.ne.jp>");
 	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
@@ -233,6 +248,7 @@ static void about_create(void)
 				      NULL, NULL, NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_button);
+	gtk_widget_grab_focus(ok_button);
 	g_signal_connect_closure
 		(G_OBJECT(ok_button), "clicked",
 		 g_cclosure_new_swap(G_CALLBACK(gtk_widget_hide_on_delete),
