@@ -35,10 +35,6 @@
 
 #include <string.h>
 #include <ctype.h>
-#if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
-#  include <wchar.h>
-#  include <wctype.h>
-#endif
 
 #include "xml.h"
 #include "addr_compl.h"
@@ -243,7 +239,7 @@ gchar *get_address_from_edit(GtkEntry *entry, gint *start_pos)
 		return NULL;
 
 #define IS_VALID_CHAR(x) \
-	(isalnum(x) || (x) == '"' || (x) == '<' || ((guchar)(x) > 0x7f))
+	(g_ascii_isalnum(x) || (x) == '"' || (x) == '<' || ((guchar)(x) > 0x7f))
 
 	/* now scan back until we hit a valid character */
 	for (; *p && !IS_VALID_CHAR(*p); p = g_utf8_next_char(p))
@@ -936,15 +932,17 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	clear_completion_cache();
 
 	/* make sure anything we typed comes in the edit box */
-	tmp_event.type       = event->type;
-	tmp_event.window     = entry->window;
-	tmp_event.send_event = TRUE;
-	tmp_event.time       = event->time;
-	tmp_event.state      = event->state;
-	tmp_event.keyval     = event->keyval;
-	tmp_event.length     = event->length;
-	tmp_event.string     = event->string;
-	gtk_widget_event(entry, (GdkEvent *)&tmp_event);
+	if (event->length > 0 && event->keyval != GDK_Escape) {
+		tmp_event.type       = event->type;
+		tmp_event.window     = entry->window;
+		tmp_event.send_event = TRUE;
+		tmp_event.time       = event->time;
+		tmp_event.state      = event->state;
+		tmp_event.keyval     = event->keyval;
+		tmp_event.length     = event->length;
+		tmp_event.string     = event->string;
+		gtk_widget_event(entry, (GdkEvent *)&tmp_event);
+	}
 
 	/* and close the completion window */
 	gtk_widget_destroy(*completion_window);
