@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,7 +76,6 @@ struct _Compose
 	GtkWidget *vbox;
 	GtkWidget *menubar;
 
-	GtkWidget *handlebox;
 	GtkWidget *toolbar;
 	GtkWidget *send_btn;
 	GtkWidget *sendl_btn;
@@ -87,6 +86,8 @@ struct _Compose
 	GtkWidget *exteditor_btn;
 	GtkWidget *linewrap_btn;
 	GtkWidget *addrbook_btn;
+	GtkWidget *prefs_common_btn;
+	GtkWidget *prefs_account_btn;
 
 	GtkWidget *vbox2;
 
@@ -131,6 +132,12 @@ struct _Compose
 
 	GtkWidget *tmpl_menu;
 
+#if USE_GTKSPELL
+        GtkWidget *spell_menu;
+        gchar     *spell_lang;
+        gboolean   check_spell;
+#endif
+
 	ComposeMode mode;
 
 	MsgInfo *targetinfo;
@@ -177,10 +184,13 @@ struct _Compose
 	GtkTextTag *sig_tag;
 
 	/* external editor */
-	gchar      *exteditor_file;
-	pid_t       exteditor_pid;
-	GIOChannel *exteditor_ch;
-	gint        exteditor_tag;
+	gchar *exteditor_file;
+	GPid   exteditor_pid;
+	guint  exteditor_tag;
+
+	guint autosave_tag;
+
+	guint lock_count;
 };
 
 struct _AttachInfo
@@ -189,10 +199,10 @@ struct _AttachInfo
 	gchar *content_type;
 	EncodingType encoding;
 	gchar *name;
-	off_t size;
+	gsize size;
 };
 
-void compose_new		(PrefsAccount	*account,
+Compose *compose_new		(PrefsAccount	*account,
 				 FolderItem	*item,
 				 const gchar	*mailto,
 				 GPtrArray	*attach_files);
@@ -218,6 +228,9 @@ void compose_entry_append	(Compose	  *compose,
 				 const gchar	  *text,
 				 ComposeEntryType  type);
 
+void compose_lock		(Compose	*compose);
+void compose_unlock		(Compose	*compose);
+
 void compose_reflect_prefs_all	(void);
 
 #endif /* __COMPOSE_H__ */
