@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@
 #include "procmsg.h"
 #include "utils.h"
 #include "gtkutils.h"
+#include "trayicon.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "prefs_folder_item.h"
@@ -404,7 +405,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("New"), renderer, "text", COL_NEW, NULL);
+		(_("New"), renderer, "text", COL_NEW,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -418,7 +421,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("Unread"), renderer, "text", COL_UNREAD, NULL);
+		(_("Unread"), renderer, "text", COL_UNREAD,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -432,7 +437,9 @@ FolderView *folderview_create(void)
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0, "ypad", 0, NULL);
 	column = gtk_tree_view_column_new_with_attributes
-		(_("#"), renderer, "text", COL_TOTAL, NULL);
+		(_("Total"), renderer, "text", COL_TOTAL,
+		 "foreground-gdk", COL_FOREGROUND,
+		 "weight", COL_BOLD, NULL);
 	gtk_tree_view_column_set_alignment(column, 1.0);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
@@ -704,6 +711,7 @@ void folderview_select_next_unread(FolderView *folderview)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
 	GtkTreeIter iter, next;
+	gboolean remember_last;
 
 	if (folderview->opened) {
 		GtkTreePath *path;
@@ -718,7 +726,10 @@ void folderview_select_next_unread(FolderView *folderview)
 			return;
 	}
 	if (folderview_find_next_unread(model, &next, &iter)) {
+		remember_last = prefs_common.remember_last_selected;
+		prefs_common.remember_last_selected = FALSE;
 		folderview_select_row(folderview, &next);
+		prefs_common.remember_last_selected = remember_last;
 		return;
 	}
 
@@ -726,8 +737,12 @@ void folderview_select_next_unread(FolderView *folderview)
 		return;
 
 	/* search again from the first row */
-	if (folderview_find_next_unread(model, &next, NULL))
+	if (folderview_find_next_unread(model, &next, NULL)) {
+		remember_last = prefs_common.remember_last_selected;
+		prefs_common.remember_last_selected = FALSE;
 		folderview_select_row(folderview, &next);
+		prefs_common.remember_last_selected = remember_last;
+	}
 }
 
 FolderItem *folderview_get_selected_item(FolderView *folderview)
@@ -895,7 +910,7 @@ static GtkWidget *label_window_create(const gchar *str)
 	gtk_container_add(GTK_CONTAINER(window), label);
 	gtk_widget_show(label);
 
-	gtk_widget_show_now(window);
+	gtk_widget_show(window);
 
 	return window;
 }
@@ -950,7 +965,7 @@ gint folderview_check_new(Folder *folder)
 	folderview = (FolderView *)folderview_list->data;
 	model = GTK_TREE_MODEL(folderview->store);
 
-	if (folder && !FOLDER_IS_LOCAL(folder)) {
+	if (folder && FOLDER_IS_REMOTE(folder)) {
 		if (!main_window_toggle_online_if_offline(folderview->mainwin))
 			return 0;
 	}
@@ -969,13 +984,14 @@ gint folderview_check_new(Folder *folder)
 		if (item->stype == F_VIRTUAL) continue;
 		if (item->no_select) continue;
 		if (folder && folder != item->folder) continue;
-		if (!folder && !FOLDER_IS_LOCAL(item->folder)) continue;
+		if (!folder && FOLDER_IS_REMOTE(item->folder)) continue;
 
 		prev_new = item->new;
 		prev_unread = item->unread;
 		folderview_scan_tree_func(item->folder, item, NULL);
 		if (folder_item_scan(item) < 0) {
-			if (folder && !FOLDER_IS_LOCAL(folder))
+			if (folder && FOLDER_IS_REMOTE(folder) &&
+			    REMOTE_FOLDER(folder)->session == NULL)
 				break;
 		}
 		folderview_update_row(folderview, &iter);
@@ -2146,6 +2162,8 @@ static void folderview_mark_all_read_cb(FolderView *folderview, guint action,
 	else {
 		procmsg_mark_all_read(item);
 		folderview_update_item(item, FALSE);
+		trayicon_set_tooltip(NULL);
+		trayicon_set_notify(FALSE);
 	}
 }
 
@@ -2501,6 +2519,8 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	procmsg_empty_trash(folder->trash);
 	statusbar_pop_all();
 	folderview_update_item(folder->trash, TRUE);
+	trayicon_set_tooltip(NULL);
+	trayicon_set_notify(FALSE);
 
 	open_path = gtk_tree_row_reference_get_path(folderview->opened);
 	if (open_path && sel_path &&
