@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,11 +42,16 @@
 #include <stdio.h>
 #include <unistd.h>
 
+#ifdef G_OS_WIN32
+#  include <windows.h>
+#endif
+
 #include "main.h"
 #include "mimeview.h"
 #include "textview.h"
 #include "imageview.h"
 #include "procmime.h"
+#include "procheader.h"
 #include "summaryview.h"
 #include "menu.h"
 #include "filesel.h"
@@ -124,7 +129,9 @@ static GtkItemFactoryEntry mimeview_popup_entries[] =
 	{N_("/_Open"),		  NULL, mimeview_launch,	  0, NULL},
 	{N_("/Open _with..."),	  NULL, mimeview_open_with,	  0, NULL},
 	{N_("/_Display as text"), NULL, mimeview_display_as_text, 0, NULL},
-	{N_("/_Save as..."),	  NULL, mimeview_save_as,	  0, NULL}
+	{N_("/_Save as..."),	  NULL, mimeview_save_as,	  0, NULL},
+	{N_("/Save _all..."),	  NULL, mimeview_save_all,	  0, NULL},
+	{N_("/_Print..."),	  NULL, mimeview_print,		  0, NULL}
 #if USE_GPGME
         ,
         {N_("/_Check signature"), NULL, mimeview_check_signature, 0, NULL}
@@ -324,7 +331,7 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	if (prefs_common.auto_check_signatures) {
 		FILE *fp;
 
-		if ((fp = fopen(file, "rb")) == NULL) {
+		if ((fp = g_fopen(file, "rb")) == NULL) {
 			FILE_OP_ERROR(file, "fopen");
 			return;
 		}
@@ -515,22 +522,13 @@ static void mimeview_show_message_part(MimeView *mimeview, MimeInfo *partinfo)
 {
 	FILE *fp;
 	const gchar *fname;
-#if USE_GPGME
-	MimeInfo *pi;
-#endif
 
 	if (!partinfo) return;
 
-#if USE_GPGME
-	for (pi = partinfo; pi && !pi->plaintextfile ; pi = pi->parent)
-		;
-	fname = pi ? pi->plaintextfile : mimeview->file;
-#else
 	fname = mimeview->file;
-#endif /* USE_GPGME */
 	if (!fname) return;
 
-	if ((fp = fopen(fname, "rb")) == NULL) {
+	if ((fp = g_fopen(fname, "rb")) == NULL) {
 		FILE_OP_ERROR(fname, "fopen");
 		return;
 	}
@@ -562,7 +560,7 @@ static void mimeview_show_image_part(MimeView *mimeview, MimeInfo *partinfo)
 		mimeview_change_view_type(mimeview, MIMEVIEW_IMAGE);
 		imageview_show_image(mimeview->imageview, partinfo, filename,
 				     prefs_common.resize_image);
-		unlink(filename);
+		g_unlink(filename);
 	}
 
 	g_free(filename);
@@ -627,7 +625,8 @@ static void mimeview_show_mime_part(MimeView *mimeview, MimeInfo *partinfo)
 		g_snprintf(buf, sizeof(buf), "[%s (%s)]\n\n",
 			   partinfo->content_type,
 			   to_human_readable(partinfo->content_size));
-	gtk_text_buffer_insert(buffer, &iter, buf, -1);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, buf, -1,
+						 "mimepart", NULL);
 
 	vbbox = gtk_vbutton_box_new();
 	gtk_box_set_spacing(GTK_BOX(vbbox), 5);
@@ -724,6 +723,17 @@ static void mimeview_change_view_type(MimeView *mimeview, MimeViewType type)
 		g_list_free(children);
 	}
 
+	switch (mimeview->type) {
+	case MIMEVIEW_IMAGE:
+		imageview_clear(mimeview->imageview);
+		break;
+	case MIMEVIEW_TEXT:
+		textview_clear(mimeview->textview);
+		break;
+	default:
+		break;
+	}
+
 	switch (type) {
 	case MIMEVIEW_IMAGE:
 		gtk_container_add(GTK_CONTAINER(mimeview->mime_vbox),
@@ -785,8 +795,7 @@ static void mimeview_selection_changed(GtkTreeSelection *selection,
 	default:
 		mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
 #if USE_GPGME
-		if (g_ascii_strcasecmp(partinfo->content_type,
-				       "application/pgp-signature") == 0)
+		if (rfc2015_is_signature_part(partinfo))
 			mimeview_show_signature_part(mimeview, partinfo);
 		else
 #endif
@@ -836,6 +845,15 @@ static gint mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 		else
 			menu_set_sensitive(mimeview->popupfactory,
 					   "/Open", TRUE);
+
+		if (partinfo && (partinfo->mime_type == MIME_TEXT ||
+				 partinfo->mime_type == MIME_TEXT_HTML ||
+				 partinfo->mime_type == MIME_MESSAGE_RFC822))
+			menu_set_sensitive(mimeview->popupfactory,
+					   "/Print...", TRUE);
+		else
+			menu_set_sensitive(mimeview->popupfactory,
+					   "/Print...", FALSE);
 #if USE_GPGME
 		menu_set_sensitive(mimeview->popupfactory,
 				   "/Check signature",
@@ -883,6 +901,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 	switch (event->keyval) {
 	case GDK_space:
+	case GDK_KP_Space:
 		if (textview_scroll_page(mimeview->textview, mod_pressed))
 			return TRUE;
 
@@ -901,6 +920,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		textview_scroll_page(mimeview->textview, TRUE);
 		return TRUE;
 	case GDK_Return:
+	case GDK_KP_Enter:
 		textview_scroll_one_line(mimeview->textview, mod_pressed);
 		return TRUE;
 	case GDK_t:
@@ -914,6 +934,8 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		break;
 	case GDK_Left:
 	case GDK_Delete:
+	case GDK_KP_Left:
+	case GDK_KP_Delete:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		break;
@@ -980,7 +1002,7 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 	if (!mimeview->drag_file) return;
 
 	gtk_selection_data_set(selection_data, selection_data->target, 8,
-			       mimeview->drag_file,
+			       (guchar *)mimeview->drag_file,
 			       strlen(mimeview->drag_file));
 }
 
@@ -1024,6 +1046,55 @@ void mimeview_save_as(MimeView *mimeview)
 	g_free(filename);
 }
 
+void mimeview_save_all(MimeView *mimeview)
+{
+	gchar *dir;
+
+	dir = filesel_select_dir(NULL);
+	if (!dir) return;
+
+	if (procmime_get_all_parts(dir, mimeview->file, mimeview->mimeinfo) < 0)
+		alertpanel_error(_("Can't save the attachments."));
+
+	g_free(dir);
+}
+
+void mimeview_print(MimeView *mimeview)
+{
+	MimeInfo *partinfo;
+
+	if (!mimeview->opened) return;
+	if (!mimeview->file) return;
+
+	partinfo = mimeview_get_selected_part(mimeview);
+	g_return_if_fail(partinfo != NULL);
+
+	if (partinfo->mime_type == MIME_MESSAGE_RFC822) {
+		gchar *filename;
+		MsgInfo *msginfo;
+		MsgFlags flags = {0, 0};
+
+		filename = procmime_get_tmp_file_name(partinfo);
+		if (procmime_get_part(filename, mimeview->file, partinfo) < 0) {
+			alertpanel_error
+				(_("Can't save the part of multipart message."));
+			g_free(filename);
+			return;
+		}
+
+		msginfo = procheader_parse_file(filename, flags, TRUE);
+		msginfo->file_path = filename;
+		filename = NULL;
+		printing_print_message
+			(msginfo, mimeview->textview->show_all_headers);
+		procmsg_msginfo_free(msginfo);
+	} else if (partinfo->mime_type == MIME_TEXT ||
+		   partinfo->mime_type == MIME_TEXT_HTML) {
+		printing_print_message_part(mimeview->messageview->msginfo,
+					    partinfo);
+	}
+}
+
 static void mimeview_launch(MimeView *mimeview)
 {
 	MimeInfo *partinfo;
@@ -1092,50 +1163,62 @@ static void mimeview_open_with(MimeView *mimeview)
 static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 			       const gchar *cmdline)
 {
-	static gchar *default_image_cmdline = "display '%s'";
-	static gchar *default_audio_cmdline = "play '%s'";
-	static gchar *default_html_cmdline = DEFAULT_BROWSER_CMD;
-	static gchar *mime_cmdline = "metamail -d -b -x -c %s '%s'";
-	gchar buf[1024];
-	gchar m_buf[1024];
-	const gchar *cmd;
-	const gchar *def_cmd;
-	const gchar *p;
+	const gchar *cmd = NULL;
+	gchar buf[BUFFSIZE];
 
-	if (cmdline) {
-		cmd = cmdline;
-		def_cmd = NULL;
-	} else if (MIME_APPLICATION_OCTET_STREAM == partinfo->mime_type) {
-		return;
-	} else if (MIME_IMAGE == partinfo->mime_type) {
-		cmd = prefs_common.mime_image_viewer;
-		def_cmd = default_image_cmdline;
-	} else if (MIME_AUDIO == partinfo->mime_type) {
-		cmd = prefs_common.mime_audio_player;
-		def_cmd = default_audio_cmdline;
-	} else if (MIME_TEXT_HTML == partinfo->mime_type) {
-		cmd = prefs_common.uri_cmd;
-		def_cmd = default_html_cmdline;
-	} else {
-		g_snprintf(m_buf, sizeof(m_buf), mime_cmdline,
-			   partinfo->content_type, "%s");
-		cmd = m_buf;
-		def_cmd = NULL;
-	}
+	if (!cmdline) {
+#ifdef G_OS_WIN32
+		DWORD dwtype;
 
-	if (cmd && (p = strchr(cmd, '%')) && *(p + 1) == 's' &&
-	    !strchr(p + 2, '%'))
-		g_snprintf(buf, sizeof(buf), cmd, filename);
-	else {
-		if (cmd)
-			g_warning(_("MIME viewer command line is invalid: `%s'"), cmd);
-		if (def_cmd)
-			g_snprintf(buf, sizeof(buf), def_cmd, filename);
-		else
+		if (g_file_test(filename, G_FILE_TEST_IS_EXECUTABLE) ||
+		    str_has_suffix_case(filename, ".scr") ||
+		    str_has_suffix_case(filename, ".pif") ||
+		    GetBinaryType(filename, &dwtype)) {
+			alertpanel_full
+				(_("Opening executable file"),
+				 _("This is an executable file. Opening executable file is restricted for security.\n"
+				   "If you want to launch it, save it to somewhere and make sure it is not an virus or something like a malicious program."),
+				 ALERT_WARNING, G_ALERTDEFAULT, FALSE,
+				 GTK_STOCK_OK, NULL, NULL);
 			return;
-	}
+		}
+		execute_open_file(filename, partinfo->content_type);
+		return;
+#else
+		if (MIME_IMAGE == partinfo->mime_type)
+			cmd = prefs_common.mime_image_viewer;
+		else if (MIME_AUDIO == partinfo->mime_type)
+			cmd = prefs_common.mime_audio_player;
+		else if (MIME_TEXT_HTML == partinfo->mime_type)
+			cmd = prefs_common.uri_cmd;
+		if (!cmd) {
+			if (prefs_common.mime_cmd) {
+				if (str_find_format_times
+					(prefs_common.mime_cmd, 's') == 2) {
+					g_snprintf(buf, sizeof(buf),
+						   prefs_common.mime_cmd,
+						   partinfo->content_type,
+						   "%s");
+					cmd = buf;
+				} else
+					cmd = prefs_common.mime_cmd;
+			} else {
+				procmime_execute_open_file
+					(filename, partinfo->content_type);
+				return;
+			}
+		}
+#endif
+	} else
+		cmd = cmdline;
 
-	execute_command_line(buf, TRUE);
+	if (cmd && str_find_format_times(cmd, 's') == 1) {
+		gchar *cmdbuf;
+		cmdbuf = g_strdup_printf(cmd, filename);
+		execute_command_line(cmdbuf, TRUE);
+		g_free(cmdbuf);
+	} else if (cmd)
+		g_warning("MIME viewer command line is invalid: '%s'", cmd);
 }
 
 #if USE_GPGME
@@ -1171,8 +1254,7 @@ static void mimeview_update_signature_info(MimeView *mimeview)
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) return;
 
-	if (g_ascii_strcasecmp(partinfo->content_type,
-			       "application/pgp-signature") == 0) {
+	if (rfc2015_is_signature_part(partinfo)) {
 		mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
 		mimeview_show_signature_part(mimeview, partinfo);
 	}
@@ -1192,7 +1274,7 @@ static void mimeview_check_signature(MimeView *mimeview)
 	while (mimeinfo->parent)
 		mimeinfo = mimeinfo->parent;
 
-	if ((fp = fopen(mimeview->file, "rb")) == NULL) {
+	if ((fp = g_fopen(mimeview->file, "rb")) == NULL) {
 		FILE_OP_ERROR(mimeview->file, "fopen");
 		return;
 	}
