@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,7 @@
 #include "imap.h"
 
 static gboolean cancelled;
+static gboolean new_account;
 static PrefsDialog dialog;
 
 static struct Basic {
@@ -96,6 +97,7 @@ static struct Receive {
 	GtkWidget *imap_frame;
 	GtkWidget *imap_auth_type_optmenu;
 	GtkWidget *imap_check_inbox_chkbtn;
+	GtkWidget *imap_filter_inbox_chkbtn;
 
 	GtkWidget *nntp_frame;
 	GtkWidget *maxarticle_spinbtn;
@@ -114,11 +116,13 @@ static struct Send {
 	GtkWidget *smtp_auth_type_optmenu;
 	GtkWidget *smtp_uid_entry;
 	GtkWidget *smtp_pass_entry;
-	/* GtkWidget *pop_bfr_smtp_chkbtn; */
+	GtkWidget *pop_bfr_smtp_chkbtn;
 } p_send;
 
 static struct Compose {
-	GtkWidget *sigfile_radiobtn;
+	GtkWidget *sig_radiobtn;
+	GtkWidget *sig_text;
+	GtkTextBuffer *sig_buffer;
 	GtkWidget *sigpath_entry;
 
 	GtkWidget *autocc_chkbtn;
@@ -127,6 +131,8 @@ static struct Compose {
 	GtkWidget *autobcc_entry;
 	GtkWidget *autoreplyto_chkbtn;
 	GtkWidget *autoreplyto_entry;
+
+	gboolean sig_modified;
 } compose;
 
 #if USE_GPGME
@@ -262,6 +268,8 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_entry, prefs_set_entry},
 	{"imap_check_inbox_only", &receive.imap_check_inbox_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"imap_filter_inbox_on_receive", &receive.imap_filter_inbox_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"imap_auth_method", &receive.imap_auth_type_optmenu,
 	 prefs_account_imap_auth_type_set_data_from_optmenu,
 	 prefs_account_imap_auth_type_set_optmenu},
@@ -287,14 +295,17 @@ static PrefsUIData ui_data[] = {
 	{"smtp_password", &p_send.smtp_pass_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 
-	/* {"pop_before_smtp", NULL, NULL, NULL}, */
+	{"pop_before_smtp", &p_send.pop_bfr_smtp_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 
 	/* Compose */
-	{"signature_type", &compose.sigfile_radiobtn,
+	{"signature_type", &compose.sig_radiobtn,
 	 prefs_account_enum_set_data_from_radiobtn,
 	 prefs_account_enum_set_radiobtn},
 	{"signature_path", &compose.sigpath_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
+	{"signature_text", &compose.sig_text,
+	 prefs_set_data_from_text, prefs_set_text},
 	{"set_autocc", &compose.autocc_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"auto_cc", &compose.autocc_entry,
@@ -407,6 +418,11 @@ static void prefs_account_select_folder_cb	(GtkWidget	*widget,
 						 gpointer	 data);
 static void prefs_account_edit_custom_header	(void);
 
+static void prefs_account_name_entry_changed_cb	(GtkWidget	*widget,
+						 gpointer	 data);
+static void prefs_account_sig_changed_cb	(GtkWidget	*widget,
+						 gpointer	 data);
+
 static gint prefs_account_deleted		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
@@ -420,7 +436,6 @@ static void prefs_account_cancel		(void);
 
 PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 {
-	gboolean new_account = FALSE;
 	static gboolean ui_registered = FALSE;
 
 	debug_print(_("Opening account preferences window...\n"));
@@ -428,6 +443,8 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 	inc_lock();
 
 	cancelled = FALSE;
+	new_account = FALSE;
+	compose.sig_modified = FALSE;
 
 	if (!ui_registered) {
 		prefs_register_ui(prefs_account_get_params(), ui_data);
@@ -455,10 +472,17 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 		PrefsAccount *def_ac;
 		gchar *buf;
 
+		g_signal_handlers_block_by_func
+			(G_OBJECT(compose.sig_buffer),
+			 G_CALLBACK(prefs_account_sig_changed_cb), NULL);
+
+		compose.sig_modified = TRUE;
 		prefs_set_dialog_to_default(prefs_account_get_params());
 		buf = g_strdup_printf(_("Account%d"), ac_prefs->account_id);
 		gtk_entry_set_text(GTK_ENTRY(basic.acname_entry), buf);
 		g_free(buf);
+		compose.sig_modified = FALSE;
+
 		def_ac = account_get_default();
 		if (def_ac) {
 			gtk_entry_set_text(GTK_ENTRY(basic.name_entry),
@@ -477,6 +501,10 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 		gtk_window_set_title(GTK_WINDOW(dialog.window),
 				     _("Preferences for new account"));
 		gtk_widget_hide(dialog.apply_btn);
+
+		g_signal_handlers_unblock_by_func
+			(G_OBJECT(compose.sig_buffer),
+			 G_CALLBACK(prefs_account_sig_changed_cb), NULL);
 	} else {
 		prefs_set_dialog(prefs_account_get_params());
 		gtk_window_set_title(GTK_WINDOW(dialog.window),
@@ -538,6 +566,13 @@ static void prefs_account_create(void)
 #endif /* USE_SSL */
 	prefs_account_advanced_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Advanced"), page++);
+
+	g_signal_connect(G_OBJECT(basic.name_entry), "changed",
+			 G_CALLBACK(prefs_account_name_entry_changed_cb), NULL);
+	g_signal_connect(G_OBJECT(basic.addr_entry), "changed",
+			 G_CALLBACK(prefs_account_name_entry_changed_cb), NULL);
+	g_signal_connect(G_OBJECT(compose.sig_buffer), "changed",
+			 G_CALLBACK(prefs_account_sig_changed_cb), NULL);
 }
 
 #define SET_ACTIVATE(menuitem) \
@@ -807,6 +842,7 @@ static void prefs_account_receive_create(void)
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
 	GtkWidget *imap_check_inbox_chkbtn;
+	GtkWidget *imap_filter_inbox_chkbtn;
 
 	GtkWidget *nntp_frame;
 	GtkWidget *maxarticle_label;
@@ -879,7 +915,7 @@ static void prefs_account_receive_create(void)
 	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
 
 	PACK_CHECK_BUTTON (vbox2, getall_chkbtn,
-			   _("Download all messages on server"));
+			   _("Download all messages (including already received) on server"));
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
@@ -959,6 +995,8 @@ static void prefs_account_receive_create(void)
 
 	PACK_CHECK_BUTTON (vbox2, imap_check_inbox_chkbtn,
 			   _("Only check INBOX on receiving"));
+	PACK_CHECK_BUTTON (vbox2, imap_filter_inbox_chkbtn,
+			   _("Filter new messages in INBOX on receiving"));
 
 	PACK_FRAME (vbox1, nntp_frame, _("News"));
 
@@ -1007,9 +1045,10 @@ static void prefs_account_receive_create(void)
 	receive.inbox_entry           = inbox_entry;
 	receive.inbox_btn             = inbox_btn;
 
-	receive.imap_frame              = imap_frame;
-	receive.imap_auth_type_optmenu  = optmenu;
-	receive.imap_check_inbox_chkbtn = imap_check_inbox_chkbtn;
+	receive.imap_frame               = imap_frame;
+	receive.imap_auth_type_optmenu   = optmenu;
+	receive.imap_check_inbox_chkbtn  = imap_check_inbox_chkbtn;
+	receive.imap_filter_inbox_chkbtn = imap_filter_inbox_chkbtn;
 
 	receive.nntp_frame             = nntp_frame;
 	receive.maxarticle_spinbtn     = maxarticle_spinbtn;
@@ -1039,7 +1078,7 @@ static void prefs_account_send_create(void)
 	GtkWidget *smtp_uid_entry;
 	GtkWidget *smtp_pass_entry;
 	GtkWidget *vbox_spc;
-	/* GtkWidget *pop_bfr_smtp_chkbtn; */
+	GtkWidget *pop_bfr_smtp_chkbtn;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -1167,11 +1206,10 @@ static void prefs_account_send_create(void)
 
 	SET_TOGGLE_SENSITIVITY (smtp_auth_chkbtn, vbox4);
 
-#if 0
+	PACK_VSPACER(vbox3, vbox_spc, VSPACING_NARROW_2);
+
 	PACK_CHECK_BUTTON (vbox3, pop_bfr_smtp_chkbtn,
 		_("Authenticate with POP3 before sending"));
-	gtk_widget_set_sensitive(pop_bfr_smtp_chkbtn, FALSE);
-#endif
 
 	p_send.date_chkbtn      = date_chkbtn;
 	p_send.msgid_chkbtn     = msgid_chkbtn;
@@ -1181,13 +1219,16 @@ static void prefs_account_send_create(void)
 	p_send.smtp_auth_type_optmenu = optmenu;
 	p_send.smtp_uid_entry         = smtp_uid_entry;
 	p_send.smtp_pass_entry        = smtp_pass_entry;
-	/* p_send.pop_bfr_smtp_chkbtn    = pop_bfr_smtp_chkbtn; */
+	p_send.pop_bfr_smtp_chkbtn    = pop_bfr_smtp_chkbtn;
 }
 
 static void prefs_account_compose_create(void)
 {
 	GtkWidget *vbox1;
 	GtkWidget *sig_vbox;
+	GtkWidget *sig_radiobtn;
+	GtkWidget *sigtext_scrwin;
+	GtkWidget *sig_text;
 	GtkWidget *sig_hbox;
 	GtkWidget *sigfile_radiobtn;
 	GtkWidget *sigcmd_radiobtn;
@@ -1213,11 +1254,34 @@ static void prefs_account_compose_create(void)
 	gtk_container_add (GTK_CONTAINER (frame), sig_vbox);
 	gtk_container_set_border_width (GTK_CONTAINER (sig_vbox), 8);
 
+	sig_radiobtn = gtk_radio_button_new_with_label
+		(NULL, _("Direct input"));
+	gtk_widget_show (sig_radiobtn);
+	gtk_box_pack_start (GTK_BOX (sig_vbox), sig_radiobtn, FALSE, FALSE, 0);
+	g_object_set_data (G_OBJECT (sig_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIG_DIRECT));
+
+	sigtext_scrwin = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (sigtext_scrwin);
+	gtk_box_pack_start (GTK_BOX (sig_vbox), sigtext_scrwin, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy
+		(GTK_SCROLLED_WINDOW (sigtext_scrwin),
+		 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+	gtk_scrolled_window_set_shadow_type
+		(GTK_SCROLLED_WINDOW (sigtext_scrwin), GTK_SHADOW_IN);
+
+	sig_text = gtk_text_view_new ();
+	gtk_widget_show (sig_text);
+	gtk_container_add (GTK_CONTAINER (sigtext_scrwin), sig_text);
+	gtk_text_view_set_editable (GTK_TEXT_VIEW (sig_text), TRUE);
+	gtk_widget_set_size_request(sig_text, DEFAULT_ENTRY_WIDTH, 60);
+
 	sig_hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (sig_hbox);
 	gtk_box_pack_start (GTK_BOX (sig_vbox), sig_hbox, FALSE, FALSE, 0);
 
-	sigfile_radiobtn = gtk_radio_button_new_with_label (NULL, _("File"));
+	sigfile_radiobtn = gtk_radio_button_new_with_label_from_widget
+		(GTK_RADIO_BUTTON(sig_radiobtn), _("File"));
 	gtk_widget_show (sigfile_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigfile_radiobtn,
 			    FALSE, FALSE, 0);
@@ -1225,7 +1289,7 @@ static void prefs_account_compose_create(void)
 			   GINT_TO_POINTER (SIG_FILE));
 
 	sigcmd_radiobtn = gtk_radio_button_new_with_label_from_widget
-		(GTK_RADIO_BUTTON(sigfile_radiobtn), _("Command output"));
+		(GTK_RADIO_BUTTON(sig_radiobtn), _("Command output"));
 	gtk_widget_show (sigcmd_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigcmd_radiobtn,
 			    FALSE, FALSE, 0);
@@ -1236,7 +1300,17 @@ static void prefs_account_compose_create(void)
 	gtk_widget_show (sigpath_entry);
 	gtk_box_pack_start (GTK_BOX (sig_vbox), sigpath_entry, TRUE, TRUE, 0);
 
-	PACK_FRAME (vbox1, frame, _("Automatically set the following addresses"));
+	SET_TOGGLE_SENSITIVITY (sig_radiobtn, sig_text);
+	SET_TOGGLE_SENSITIVITY (sigfile_radiobtn, sigpath_entry);
+	SET_TOGGLE_SENSITIVITY (sigcmd_radiobtn, sigpath_entry);
+	SET_TOGGLE_SENSITIVITY_REV (sig_radiobtn, sigpath_entry);
+	SET_TOGGLE_SENSITIVITY_REV (sigfile_radiobtn, sig_text);
+	SET_TOGGLE_SENSITIVITY_REV (sigcmd_radiobtn, sig_text);
+	gtk_widget_set_sensitive (sig_text, TRUE);
+	gtk_widget_set_sensitive (sigpath_entry, FALSE);
+
+	PACK_FRAME (vbox1, frame,
+		    _("Automatically set the following addresses"));
 
 	table =  gtk_table_new (3, 2, FALSE);
 	gtk_widget_show (table);
@@ -1284,9 +1358,11 @@ static void prefs_account_compose_create(void)
 
 	SET_TOGGLE_SENSITIVITY (autoreplyto_chkbtn, autoreplyto_entry);
 
+	compose.sig_radiobtn  = sig_radiobtn;
+	compose.sig_text      = sig_text;
+	compose.sigpath_entry = sigpath_entry;
 
-	compose.sigfile_radiobtn = sigfile_radiobtn;
-	compose.sigpath_entry    = sigpath_entry;
+	compose.sig_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(sig_text));
 
 	compose.autocc_chkbtn      = autocc_chkbtn;
 	compose.autocc_entry       = autocc_entry;
@@ -1324,9 +1400,9 @@ static void prefs_account_privacy_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
 
 	PACK_CHECK_BUTTON (vbox2, default_sign_chkbtn,
-			   _("Sign message by default"));
+			   _("PGP sign message by default"));
 	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
-			   _("Encrypt message by default"));
+			   _("PGP encrypt message by default"));
 	PACK_CHECK_BUTTON (vbox2, encrypt_reply_chkbtn,
 			   _("Encrypt when replying to encrypted message"));
 	PACK_CHECK_BUTTON (vbox2, ascii_armored_chkbtn,
@@ -1875,6 +1951,42 @@ static void prefs_account_edit_custom_header(void)
 	prefs_custom_header_open(prefs_account_get_tmp_prefs());
 }
 
+static void prefs_account_name_entry_changed_cb(GtkWidget *widget,
+						gpointer data)
+{
+	PrefsAccount *tmp_ac_prefs;
+
+	tmp_ac_prefs = prefs_account_get_tmp_prefs();
+
+	if (new_account && !compose.sig_modified) {
+		gchar *sig;
+		GtkTextIter iter;
+
+		g_signal_handlers_block_by_func
+			(G_OBJECT(compose.sig_buffer),
+			 G_CALLBACK(prefs_account_sig_changed_cb), NULL);
+
+		sig = g_strdup_printf
+			("%s <%s>\n",
+			 gtk_entry_get_text(GTK_ENTRY(basic.name_entry)),
+			 gtk_entry_get_text(GTK_ENTRY(basic.addr_entry)));
+		gtk_text_buffer_set_text(compose.sig_buffer, "", 0);
+		gtk_text_buffer_get_start_iter(compose.sig_buffer, &iter);
+		gtk_text_buffer_insert(compose.sig_buffer, &iter, sig, -1);
+		g_free(sig);
+
+		g_signal_handlers_unblock_by_func
+			(G_OBJECT(compose.sig_buffer),
+			 G_CALLBACK(prefs_account_sig_changed_cb), NULL);
+
+	}
+}
+
+static void prefs_account_sig_changed_cb(GtkWidget *widget, gpointer data)
+{
+	compose.sig_modified = TRUE;
+}
+
 static void prefs_account_enum_set_data_from_radiobtn(PrefParam *pparam)
 {
 	PrefsUIData *ui_data;
@@ -2117,12 +2229,16 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_show(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
-		if (!tmp_ac_prefs->account_name) {
+		gtk_widget_set_sensitive(p_send.pop_bfr_smtp_chkbtn, FALSE);
+
+		if (new_account) {
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(receive.recvatgetall_chkbtn),
 				 FALSE);
 		}
 
+		gtk_widget_set_sensitive(p_send.pop_bfr_smtp_chkbtn, FALSE);
+
 #if USE_SSL
 		gtk_widget_hide(ssl.pop_frame);
 		gtk_widget_hide(ssl.imap_frame);
@@ -2153,12 +2269,14 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, FALSE);
 
-		if (!tmp_ac_prefs->account_name) {
+		if (new_account) {
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(receive.recvatgetall_chkbtn),
 				 TRUE);
 		}
 
+		gtk_widget_set_sensitive(p_send.pop_bfr_smtp_chkbtn, FALSE);
+
 #if USE_SSL
 		gtk_widget_hide(ssl.pop_frame);
 		gtk_widget_hide(ssl.imap_frame);
@@ -2189,12 +2307,14 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
-		if (!tmp_ac_prefs->account_name) {
+		if (new_account) {
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(receive.recvatgetall_chkbtn),
 				 FALSE);
 		}
 
+		gtk_widget_set_sensitive(p_send.pop_bfr_smtp_chkbtn, FALSE);
+
 #if USE_SSL
 		gtk_widget_hide(ssl.pop_frame);
 		gtk_widget_show(ssl.imap_frame);
@@ -2226,12 +2346,14 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
-		if (!tmp_ac_prefs->account_name) {
+		if (new_account) {
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(receive.recvatgetall_chkbtn),
 				 TRUE);
 		}
 
+		gtk_widget_set_sensitive(p_send.pop_bfr_smtp_chkbtn, TRUE);
+
 #if USE_SSL
 		gtk_widget_show(ssl.pop_frame);
 		gtk_widget_hide(ssl.imap_frame);
