@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -437,6 +437,26 @@ void textview_reflect_prefs(TextView *textview)
 					 prefs_common.textview_cursor_visible);
 }
 
+static const gchar *textview_get_src_encoding(TextView *textview,
+					      MimeInfo *mimeinfo)
+{
+	const gchar *charset;
+
+	if (textview->messageview->forced_charset)
+		charset = textview->messageview->forced_charset;
+	else if (!textview->messageview->new_window &&
+		 prefs_common.force_charset)
+		charset = prefs_common.force_charset;
+	else if (mimeinfo->charset)
+		charset = mimeinfo->charset;
+	else if (prefs_common.default_encoding)
+		charset = prefs_common.default_encoding;
+	else
+		charset = NULL;
+
+	return charset;
+}
+
 void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 			   const gchar *file)
 {
@@ -445,7 +465,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 	GtkTextMark *mark;
 	GtkTextIter iter;
 	FILE *fp;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 
 	buffer = gtk_text_view_get_buffer(text);
@@ -455,15 +475,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		return;
 	}
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
-
+	charset = textview_get_src_encoding(textview, mimeinfo);
 	textview_set_font(textview, charset);
 	textview_clear(textview);
 
@@ -477,7 +489,19 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
 	}
 
+#if USE_GPGME
+	if (textview->messageview->msginfo->encinfo &&
+	    textview->messageview->msginfo->encinfo->decryption_failed) {
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, _("This message is encrypted, but its decryption failed.\n"),
+			 -1, "error", "mimepart", NULL);
+	}
+#endif
+
 	textview_add_parts(textview, mimeinfo, fp);
+
 #if USE_GPGME
 	if (textview->messageview->msginfo->encinfo &&
 	    textview->messageview->msginfo->encinfo->sigstatus)
@@ -496,7 +520,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gchar buf[BUFFSIZE];
 	const gchar *boundary = NULL;
 	gint boundary_len = 0;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 	gboolean is_rfc822_part = FALSE;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
@@ -518,14 +542,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		boundary_len = strlen(boundary);
 	}
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
+	charset = textview_get_src_encoding(textview, mimeinfo);
 
 	if (!boundary && mimeinfo->mime_type == MIME_TEXT) {
 		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
@@ -605,7 +622,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gchar buf[BUFFSIZE];
 	const gchar *boundary = NULL;
 	gint boundary_len = 0;
-	const gchar *charset = NULL;
+	const gchar *charset;
 	GPtrArray *headers = NULL;
 
 	g_return_if_fail(mimeinfo != NULL);
@@ -629,14 +646,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	while (fgets(buf, sizeof(buf), fp) != NULL)
 		if (buf[0] == '\r' || buf[0] == '\n') break;
 
-	if (textview->messageview->forced_charset)
-		charset = textview->messageview->forced_charset;
-	else if (prefs_common.force_charset)
-		charset = prefs_common.force_charset;
-	else if (mimeinfo->charset)
-		charset = mimeinfo->charset;
-	else if (prefs_common.default_encoding)
-		charset = prefs_common.default_encoding;
+	charset = textview_get_src_encoding(textview, mimeinfo);
 
 	if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 		headers = textview_scan_header(textview, fp, charset);
@@ -1258,6 +1268,10 @@ void textview_clear(TextView *textview)
 	buffer = gtk_text_view_get_buffer(text);
 	gtk_text_buffer_set_text(buffer, "", -1);
 
+	/* workaround for the assertion failure in
+	   gtk_text_view_validate_onscreen() */
+	text->vadjustment->value = 0.0;
+
 	STATUSBAR_POP(textview);
 	textview_uri_list_remove_all(textview->uri_list);
 	textview->uri_list = NULL;
